print("import: 'compas_fab'")
import compas_fab

print("import: 'compas_fab.backends'")
import compas_fab.backends

print("import: 'compas_fab.robots'")
import compas_fab.robots

print("import: 'compas_fab.rhino'")
import compas_fab.rhino

print("import: 'compas_fab.blender'")
import compas_fab.blender

